from setuptools import setup

setup(name='jam65st_probability',
      version='1.0',
      description='Gaussian distributions',
      author='Jaime Mendez',
      author_email='jam65st@gmail.com',
      packages=['jam65st_probability'],
      zip_safe=False)
